/******  STM32F103xx  ******/

/* Manual: RM008, Doc ID 13902 Rev 10  
STM32F101xx, 
STM32F102xx, 
STM32F103xx, 
STM32F105xx
STM32F107xx
*/

#ifndef STM32F103_INCLUDED
#define STM32F103_INCLUDED

/* AHB Peripherals */
#define USBOTG  0x50000000 // USB_OTG_FS Section 26.14.6 on page 791
#define ETH     0x40028000 // Ethernet   Section 27.8.5 on page 953
#define CRC     0x40023000 // Crc        Section 3.4.4 on page 53
#define FLASH   0x40022000 // Flash memory interface
#define RCC     0x40021000 // Reset and clock control Section 6.3.11 on page 103
#define DMA2    0x40020400 // DMA2       Section 10.4.7 on page 199
#define DMA1    0x40020000 // DMA1       Section 10.4.7 on page 199
#define SDIO    0x40018000 // SDIO       Section 20.9.16 on page 513
#define FSMC    0xA0000000 // Memory Controller Section 19 on page 412

/* APB2 Peripherals */
#define ADC3    0x40013C00 // ADC3 Section 11.12.15 on page 234
#define USART1  0x40013800 // USART1  Section 25.6.8 on page 707
#define TIM8    0x40013400 // TIM8 timer Section 13.4.21 on page 320
#define SPI1    0x40013000 // SPI1 Section 23.5 on page 627
#define TIM1    0x40012C00 // TIM1 timer Section 13.4.21 on page 320
#define ADC2    0x40012800 // ADC2 Section 11.12.15 on page 234
#define ADC1    0x40012400 // ADC1 Section 11.12.15 on page 234
#define GPIOG   0x40012000 // GPIO Port G Section 8.5 on page 169
#define GPIOF   0x40011C00 // GPIO Port F Section 8.5 on page 169
#define GPIOE   0x40011800 // GPIO Port E Section 8.5 on page 169
#define GPIOD   0x40011400 // GPIO Port D Section 8.5 on page 169
#define GPIOC   0x40011000 // GPIO Port C Section 8.5 on page 169
#define GPIOB   0x40010C00 // GPIO Port B Section 8.5 on page 169
#define GPIOA   0x40010800 // GPIO Port A Section 8.5 on page 169
#define EXTI    0x40010400 // EXTI Section 9.3.7 on page 183
#define AFIO    0x40010000 // AFIO

/* APB1 Peripherals */
#define DAC     0x40007400 // DAC Section 12.5.14 on page 255
#define PWR     0x40007000 // Power control PWR Section 4.4.3 on page 66
#define BKP     0x40006C00 // Backup registers (BKP) Section 5.4.5 on page 71
#define CAN1    0x40006400 // CAN1 Section 22.9.5 on page 586
#define CAN2    0x40006800 // CAN2 Section 22.9.5 on page 586
#define SHRAM   0x40006000 // Shared USB/CAN SRAM 512 bytes
#define USB     0x40005C00 // USB device FS registers Section 21.5.4 on page 543
#define I2C2    0x40005800 // I2C2 Section 24.6.10 on page 665
#define I2C1    0x40005400 // I2C1 Section 24.6.10 on page 665
#define UART5   0x40005000 // UART5 Section 25.6.8 on page 707
#define UART4   0x40004C00 // UART4 Section 25.6.8 on page 707
#define USART3  0x40004800 // USART3 Section 25.6.8 on page 707
#define USART2  0x40004400 // USART2 Section 25.6.8 on page 707
#define SPI3    0x40003C00 // SPI3/I2S Section 23.5 on page 627
#define SPI2    0x40003800 // SPI2/I2S Section 23.5 on page 627
#define IWDG    0x40003000 // Independent watchdog (IWDG) Section 17.4.5 on page 406
#define WWDG    0x40002C00 // Window watchdog (WWDG) Section 18.6.4 on page 411
#define RTC     0x40002800 // RTC Section 16.4.7 on page 401
#define TIM7    0x40001400 // TIM7 timer Section 15.4.9 on page 390
#define TIM6    0x40001000 // TIM6 timer Section 15.4.9 on page 390
#define TIM5    0x40000C00 // TIM5 timer Section 14.4.19 on page 377
#define TIM4    0x40000800 // TIM4 timer Section 14.4.19 on page 377
#define TIM3    0x40000400 // TIM3 timer Section 14.4.19 on page 377
#define TIM2    0x40000000 // TIM2 timer


/************* CRC registers ******************/
#define CRC_DR       (*((volatile dword *) (CRC +    0)))   // CRC Data register
#define CRC_IDR      (*((volatile dword *) (CRC +    4)))   // Independent data register
#define CRC_CR       (*((volatile dword *) (CRC +    8)))   // CRC Control register
#define CRC_INIT     (*((volatile dword *) (CRC + 0x10)))   // Initial CRC value
#define CRC_POL      (*((volatile dword *) (CRC + 0x14)))   // CRC polynomial


/************* Power control (PWR) ***********/
#define PWR_CR       (*((volatile dword *) (PWR + 0)))
#define PWR_CSR      (*((volatile dword *) (PWR + 4)))


/************* BKP registers *****************/
#define BKP_DR1      (*((volatile word *) (BKP +    4)))
#define BKP_DR2      (*((volatile word *) (BKP +    8)))
#define BKP_DR3      (*((volatile word *) (BKP + 0x0C)))
#define BKP_DR4      (*((volatile word *) (BKP + 0x10)))
#define BKP_DR5      (*((volatile word *) (BKP + 0x14)))
#define BKP_DR6      (*((volatile word *) (BKP + 0x18)))
#define BKP_DR7      (*((volatile word *) (BKP + 0x1C)))
#define BKP_DR8      (*((volatile word *) (BKP + 0x20)))
#define BKP_DR9      (*((volatile word *) (BKP + 0x24)))
#define BKP_DR10     (*((volatile word *) (BKP + 0x28)))

#define BKP_RTCCR    (*((volatile word *) (BKP + 0x2C)))
#define BKP_CR       (*((volatile word *) (BKP + 0x30)))
#define BKP_CSR      (*((volatile word *) (BKP + 0x34)))

#define BKP_DR11     (*((volatile word *) (BKP + 0x40)))
#define BKP_DR12     (*((volatile word *) (BKP + 0x44)))
#define BKP_DR13     (*((volatile word *) (BKP + 0x48)))
#define BKP_DR14     (*((volatile word *) (BKP + 0x4C)))
#define BKP_DR15     (*((volatile word *) (BKP + 0x50)))
#define BKP_DR16     (*((volatile word *) (BKP + 0x54)))
#define BKP_DR17     (*((volatile word *) (BKP + 0x58)))
#define BKP_DR18     (*((volatile word *) (BKP + 0x5C)))
#define BKP_DR19     (*((volatile word *) (BKP + 0x60)))
#define BKP_DR20     (*((volatile word *) (BKP + 0x64)))
#define BKP_DR21     (*((volatile word *) (BKP + 0x68)))
#define BKP_DR22     (*((volatile word *) (BKP + 0x6C)))
#define BKP_DR23     (*((volatile word *) (BKP + 0x70)))
#define BKP_DR24     (*((volatile word *) (BKP + 0x74)))
#define BKP_DR25     (*((volatile word *) (BKP + 0x78)))
#define BKP_DR26     (*((volatile word *) (BKP + 0x7C)))
#define BKP_DR27     (*((volatile word *) (BKP + 0x80)))
#define BKP_DR28     (*((volatile word *) (BKP + 0x84)))
#define BKP_DR29     (*((volatile word *) (BKP + 0x88)))
#define BKP_DR30     (*((volatile word *) (BKP + 0x8C)))
#define BKP_DR31     (*((volatile word *) (BKP + 0x90)))
#define BKP_DR32     (*((volatile word *) (BKP + 0x94)))
#define BKP_DR33     (*((volatile word *) (BKP + 0x98)))
#define BKP_DR34     (*((volatile word *) (BKP + 0x9C)))
#define BKP_DR35     (*((volatile word *) (BKP + 0xA0)))
#define BKP_DR36     (*((volatile word *) (BKP + 0xA4)))
#define BKP_DR37     (*((volatile word *) (BKP + 0xA8)))
#define BKP_DR38     (*((volatile word *) (BKP + 0xAC)))
#define BKP_DR39     (*((volatile word *) (BKP + 0xB0)))
#define BKP_DR40     (*((volatile word *) (BKP + 0xB4)))
#define BKP_DR41     (*((volatile word *) (BKP + 0xB8)))
#define BKP_DR42     (*((volatile word *) (BKP + 0xBC)))


/************* Reset and clock control (RCC) ************/
#define RCC_CR       (*((volatile dword *) (RCC + 0)))
#define RCC_CFGR     (*((volatile dword *) (RCC + 4)))
#define RCC_CIR      (*((volatile dword *) (RCC + 8)))
#define RCC_APB2RSTR (*((volatile dword *) (RCC + 0x0C)))
#define RCC_APB1RSTR (*((volatile dword *) (RCC + 0x10)))
#define RCC_AHBENR   (*((volatile dword *) (RCC + 0x14)))
#define RCC_APB2ENR  (*((volatile dword *) (RCC + 0x18)))
#define RCC_APB1ENR  (*((volatile dword *) (RCC + 0x1C)))
#define RCC_BDCR     (*((volatile dword *) (RCC + 0x20)))
#define RCC_CSR      (*((volatile dword *) (RCC + 0x24)))
#define RCC_AHBRSTR  (*((volatile dword *) (RCC + 0x28)))
#define RCC_CFGR2    (*((volatile dword *) (RCC + 0x2C)))


/************* GPIO and AFIO register ******************/
#define GPIOA_CRL   (*((volatile dword *) (GPIOA + 0x00)))
#define GPIOA_CRH   (*((volatile dword *) (GPIOA + 0x04)))
#define GPIOA_IDR   (*((volatile dword *) (GPIOA + 0x08)))
#define GPIOA_ODR   (*((volatile dword *) (GPIOA + 0x0C))) 
#define GPIOA_BSRR  (*((volatile dword *) (GPIOA + 0x10)))
#define GPIOA_BRR   (*((volatile dword *) (GPIOA + 0x14)))
#define GPIOA_LCKR  (*((volatile dword *) (GPIOA + 0x18)))

#define GPIOB_CRL   (*((volatile dword *) (GPIOB + 0x00)))
#define GPIOB_CRH   (*((volatile dword *) (GPIOB + 0x04)))
#define GPIOB_IDR   (*((volatile dword *) (GPIOB + 0x08)))
#define GPIOB_ODR   (*((volatile dword *) (GPIOB + 0x0C))) 
#define GPIOB_BSRR  (*((volatile dword *) (GPIOB + 0x10)))
#define GPIOB_BRR   (*((volatile dword *) (GPIOB + 0x14)))
#define GPIOB_LCKR  (*((volatile dword *) (GPIOB + 0x18)))

#define GPIOC_CRL   (*((volatile dword *) (GPIOC + 0x00)))
#define GPIOC_CRH   (*((volatile dword *) (GPIOC + 0x04)))
#define GPIOC_IDR   (*((volatile dword *) (GPIOC + 0x08)))
#define GPIOC_ODR   (*((volatile dword *) (GPIOC + 0x0C))) 
#define GPIOC_BSRR  (*((volatile dword *) (GPIOC + 0x10)))
#define GPIOC_BRR   (*((volatile dword *) (GPIOC + 0x14)))
#define GPIOC_LCKR  (*((volatile dword *) (GPIOC + 0x18)))

#define GPIOD_CRL   (*((volatile dword *) (GPIOD + 0x00)))
#define GPIOD_CRH   (*((volatile dword *) (GPIOD + 0x04)))
#define GPIOD_IDR   (*((volatile dword *) (GPIOD + 0x08)))
#define GPIOD_ODR   (*((volatile dword *) (GPIOD + 0x0C))) 
#define GPIOD_BSRR  (*((volatile dword *) (GPIOD + 0x10)))
#define GPIOD_BRR   (*((volatile dword *) (GPIOD + 0x14)))
#define GPIOD_LCKR  (*((volatile dword *) (GPIOD + 0x18)))

#define GPIOE_CRL   (*((volatile dword *) (GPIOE + 0x00)))
#define GPIOE_CRH   (*((volatile dword *) (GPIOE + 0x04)))
#define GPIOE_IDR   (*((volatile dword *) (GPIOE + 0x08)))
#define GPIOE_ODR   (*((volatile dword *) (GPIOE + 0x0C))) 
#define GPIOE_BSRR  (*((volatile dword *) (GPIOE + 0x10)))
#define GPIOE_BRR   (*((volatile dword *) (GPIOE + 0x14)))
#define GPIOE_LCKR  (*((volatile dword *) (GPIOE + 0x18)))

#define GPIOF_CRL   (*((volatile dword *) (GPIOF + 0x00)))
#define GPIOF_CRH   (*((volatile dword *) (GPIOF + 0x04)))
#define GPIOF_IDR   (*((volatile dword *) (GPIOF + 0x08)))
#define GPIOF_ODR   (*((volatile dword *) (GPIOF + 0x0C))) 
#define GPIOF_BSRR  (*((volatile dword *) (GPIOF + 0x10)))
#define GPIOF_BRR   (*((volatile dword *) (GPIOF + 0x14)))
#define GPIOF_LCKR  (*((volatile dword *) (GPIOF + 0x18)))

#define GPIOG_CRL   (*((volatile dword *) (GPIOG + 0x00)))
#define GPIOG_CRH   (*((volatile dword *) (GPIOG + 0x04)))
#define GPIOG_IDR   (*((volatile dword *) (GPIOG + 0x08)))
#define GPIOG_ODR   (*((volatile dword *) (GPIOG + 0x0C))) 
#define GPIOG_BSRR  (*((volatile dword *) (GPIOG + 0x10)))
#define GPIOG_BRR   (*((volatile dword *) (GPIOG + 0x14)))
#define GPIOG_LCKR  (*((volatile dword *) (GPIOG + 0x18)))

#define AFIO_EVCR    (*((volatile dword *) (AFIO + 0x00)))
#define AFIO_MAPR    (*((volatile dword *) (AFIO + 0x04)))
#define AFIO_EXTICR1 (*((volatile dword *) (AFIO + 0x08)))
#define AFIO_EXTICR2 (*((volatile dword *) (AFIO + 0x0C)))
#define AFIO_EXTICR3 (*((volatile dword *) (AFIO + 0x10)))
#define AFIO_EXTICR4 (*((volatile dword *) (AFIO + 0x14)))


/************* Extended interrupts and events controller (EXTI) */
#define EXTI_IMR    (*((volatile dword *) (EXTI + 0x0)))
#define EXTI_EMR    (*((volatile dword *) (EXTI + 0x4)))
#define EXTI_RTSR   (*((volatile dword *) (EXTI + 0x8)))
#define EXTI_FTSR   (*((volatile dword *) (EXTI + 0xC)))
#define EXTI_SWIER  (*((volatile dword *) (EXTI + 0x10)))
#define EXTI_PR     (*((volatile dword *) (EXTI + 0x14)))


/************* DMA ****************************/
#define DMA1_ISR     (*(volatile unsigned long*) 0x40020000)
#define DMA1_IFCR    (*(volatile unsigned long*) 0x40020004)

#define DMA1_CCR1    (*(volatile unsigned long*) 0x40020008)
#define DMA1_CNDTR1  (*(volatile unsigned long*) 0x4002000C)
#define DMA1_CPAR1   (*(volatile unsigned long*) 0x40020010)
#define DMA1_CMAR1   (*(volatile unsigned long*) 0x40020014)

#define DMA1_CCR2    (*(volatile unsigned long*) 0x4002001C)
#define DMA1_CNDTR2  (*(volatile unsigned long*) 0x40020020)
#define DMA1_CPAR2   (*(volatile unsigned long*) 0x40020024)
#define DMA1_CMAR2   (*(volatile unsigned long*) 0x40020028)

#define DMA1_CCR3    (*(volatile unsigned long*) 0x40020030)
#define DMA1_CNDTR3  (*(volatile unsigned long*) 0x40020034)
#define DMA1_CPAR3   (*(volatile unsigned long*) 0x40020038)
#define DMA1_CMAR3   (*(volatile unsigned long*) 0x4002003C)

#define DMA1_CCR4    (*(volatile unsigned long*) 0x40020044)
#define DMA1_CNDTR4  (*(volatile unsigned long*) 0x40020048)
#define DMA1_CPAR4   (*(volatile unsigned long*) 0x4002004C)
#define DMA1_CMAR4   (*(volatile unsigned long*) 0x40020050)

#define DMA1_CCR5    (*(volatile unsigned long*) 0x40020058)
#define DMA1_CNDTR5  (*(volatile unsigned long*) 0x4002005C)
#define DMA1_CPAR5   (*(volatile unsigned long*) 0x40020060)
#define DMA1_CMAR5   (*(volatile unsigned long*) 0x40020064)

#define DMA1_CCR6    (*(volatile unsigned long*) 0x4002006C)
#define DMA1_CNDTR6  (*(volatile unsigned long*) 0x40020070)
#define DMA1_CPAR6   (*(volatile unsigned long*) 0x40020074)
#define DMA1_CMAR6   (*(volatile unsigned long*) 0x40020078)

#define DMA1_CCR7    (*(volatile unsigned long*) 0x40020080)
#define DMA1_CNDTR7  (*(volatile unsigned long*) 0x40020084)
#define DMA1_CPAR7   (*(volatile unsigned long*) 0x40020088)
#define DMA1_CMAR7   (*(volatile unsigned long*) 0x4002008C)


#define DMA2_ISR     (*(volatile unsigned long*) 0x40020400)
#define DMA2_IFCR    (*(volatile unsigned long*) 0x40020404)

#define DMA2_CCR1    (*(volatile unsigned long*) 0x40020408)
#define DMA2_CNDTR1  (*(volatile unsigned long*) 0x4002040C)
#define DMA2_CPAR1   (*(volatile unsigned long*) 0x40020410)
#define DMA2_CMAR1   (*(volatile unsigned long*) 0x40020414)

#define DMA2_CCR2    (*(volatile unsigned long*) 0x4002041C)
#define DMA2_CNDTR2  (*(volatile unsigned long*) 0x40020420)
#define DMA2_CPAR2   (*(volatile unsigned long*) 0x40020424)
#define DMA2_CMAR2   (*(volatile unsigned long*) 0x40020428)

#define DMA2_CCR3    (*(volatile unsigned long*) 0x40020430)
#define DMA2_CNDTR3  (*(volatile unsigned long*) 0x40020434)
#define DMA2_CPAR3   (*(volatile unsigned long*) 0x40020438)
#define DMA2_CMAR3   (*(volatile unsigned long*) 0x4002043C)

#define DMA2_CCR4    (*(volatile unsigned long*) 0x40020444)
#define DMA2_CNDTR4  (*(volatile unsigned long*) 0x40020448)
#define DMA2_CPAR4   (*(volatile unsigned long*) 0x4002044C)
#define DMA2_CMAR4   (*(volatile unsigned long*) 0x40020450)

#define DMA2_CCR5    (*(volatile unsigned long*) 0x40020458)
#define DMA2_CNDTR5  (*(volatile unsigned long*) 0x4002045C)
#define DMA2_CPAR5   (*(volatile unsigned long*) 0x40020460)
#define DMA2_CMAR5   (*(volatile unsigned long*) 0x40020464)


/************** ADC register *************************/
#define ADC1_SR     (*((volatile dword *) (ADC1 + 0x0)))
#define ADC1_CR1    (*((volatile dword *) (ADC1 + 0x4)))
#define ADC1_CR2    (*((volatile dword *) (ADC1 + 0x8)))
#define ADC1_SMPR1  (*((volatile dword *) (ADC1 + 0xC)))
#define ADC1_SMPR2  (*((volatile dword *) (ADC1 + 0x10)))
#define ADC1_JOFR1  (*((volatile dword *) (ADC1 + 0x14)))
#define ADC1_JOFR2  (*((volatile dword *) (ADC1 + 0x18)))
#define ADC1_JOFR3  (*((volatile dword *) (ADC1 + 0x1C)))
#define ADC1_JOFR4  (*((volatile dword *) (ADC1 + 0x20)))
#define ADC1_HTR    (*((volatile dword *) (ADC1 + 0x24)))
#define ADC1_LTR    (*((volatile dword *) (ADC1 + 0x28)))
#define ADC1_SQR1   (*((volatile dword *) (ADC1 + 0x2C)))
#define ADC1_SQR2   (*((volatile dword *) (ADC1 + 0x30)))
#define ADC1_SQR3   (*((volatile dword *) (ADC1 + 0x34)))
#define ADC1_JSQR   (*((volatile dword *) (ADC1 + 0x38)))
#define ADC1_JDR1   (*((volatile dword *) (ADC1 + 0x3C)))
#define ADC1_JDR2   (*((volatile dword *) (ADC1 + 0x40)))
#define ADC1_JDR3   (*((volatile dword *) (ADC1 + 0x44)))
#define ADC1_JDR4   (*((volatile dword *) (ADC1 + 0x48)))
#define ADC1_DR     (*((volatile dword *) (ADC1 + 0x4C)))

#define ADC2_SR     (*((volatile dword *) (ADC2 + 0x0)))
#define ADC2_CR1    (*((volatile dword *) (ADC2 + 0x4)))
#define ADC2_CR2    (*((volatile dword *) (ADC2 + 0x8)))
#define ADC2_SMPR1  (*((volatile dword *) (ADC2 + 0xC)))
#define ADC2_SMPR2  (*((volatile dword *) (ADC2 + 0x10)))
#define ADC2_JOFR1  (*((volatile dword *) (ADC2 + 0x14)))
#define ADC2_JOFR2  (*((volatile dword *) (ADC2 + 0x18)))
#define ADC2_JOFR3  (*((volatile dword *) (ADC2 + 0x1C)))
#define ADC2_JOFR4  (*((volatile dword *) (ADC2 + 0x20)))
#define ADC2_HTR    (*((volatile dword *) (ADC2 + 0x24)))
#define ADC2_LTR    (*((volatile dword *) (ADC2 + 0x28)))
#define ADC2_SQR1   (*((volatile dword *) (ADC2 + 0x2C)))
#define ADC2_SQR2   (*((volatile dword *) (ADC2 + 0x30)))
#define ADC2_SQR3   (*((volatile dword *) (ADC2 + 0x34)))
#define ADC2_JSQR   (*((volatile dword *) (ADC2 + 0x38)))
#define ADC2_JDR1   (*((volatile dword *) (ADC2 + 0x3C)))
#define ADC2_JDR2   (*((volatile dword *) (ADC2 + 0x40)))
#define ADC2_JDR3   (*((volatile dword *) (ADC2 + 0x44)))
#define ADC2_JDR4   (*((volatile dword *) (ADC2 + 0x48)))
#define ADC2_DR     (*((volatile dword *) (ADC2 + 0x4C)))


/************** Digital-to-analog converter (DAC1) *****/
#define DAC_CR      (*((volatile dword *) (DAC + 0x0)))
#define DAC_SWTRIGR (*((volatile dword *) (DAC + 0x4)))
#define DAC_DHR12R1 (*((volatile dword *) (DAC + 0x8)))
#define DAC_DHR12L1 (*((volatile dword *) (DAC + 0xC)))
#define DAC_DHR8R1  (*((volatile dword *) (DAC + 0x10)))
#define DAC_DHR12R2 (*((volatile dword *) (DAC + 0x14)))
#define DAC_DHR12L2 (*((volatile dword *) (DAC + 0x18)))
#define DAC_DHR8R2  (*((volatile dword *) (DAC + 0x1C)))
#define DAC_DHR12RD (*((volatile dword *) (DAC + 0x20)))
#define DAC_DHR12LD (*((volatile dword *) (DAC + 0x24)))
#define DAC_DHR8RD  (*((volatile dword *) (DAC + 0x28)))
#define DAC_DOR1    (*((volatile dword *) (DAC + 0x2C)))
#define DAC_DOR2    (*((volatile dword *) (DAC + 0x30)))
#define DAC_SR      (*((volatile dword *) (DAC + 0x34)))


/************** Advanced-control timers (TIM1) ********/
#define TIM1_CR1    (*((volatile dword *) (TIM1 + 0x0)))
#define TIM1_CR2    (*((volatile dword *) (TIM1 + 0x4)))
#define TIM1_SMCR   (*((volatile dword *) (TIM1 + 0x8)))
#define TIM1_DIER   (*((volatile dword *) (TIM1 + 0xC)))
#define TIM1_SR     (*((volatile dword *) (TIM1 + 0x10)))
#define TIM1_EGR    (*((volatile dword *) (TIM1 + 0x14)))
#define TIM1_CCMR1  (*((volatile dword *) (TIM1 + 0x18)))
#define TIM1_CCMR2  (*((volatile dword *) (TIM1 + 0x1C)))
#define TIM1_CCER   (*((volatile dword *) (TIM1 + 0x20)))
#define TIM1_CNT    (*((volatile dword *) (TIM1 + 0x24)))
#define TIM1_PSC    (*((volatile dword *) (TIM1 + 0x28)))
#define TIM1_ARR    (*((volatile dword *) (TIM1 + 0x2C)))
#define TIM1_RCR    (*((volatile dword *) (TIM1 + 0x30)))
#define TIM1_CCR1   (*((volatile dword *) (TIM1 + 0x34)))
#define TIM1_CCR2   (*((volatile dword *) (TIM1 + 0x38)))
#define TIM1_CCR3   (*((volatile dword *) (TIM1 + 0x3C)))
#define TIM1_CCR4   (*((volatile dword *) (TIM1 + 0x40)))
#define TIM1_BDTR   (*((volatile dword *) (TIM1 + 0x44)))
#define TIM1_DCR    (*((volatile dword *) (TIM1 + 0x48)))
#define TIM1_DMAR   (*((volatile dword *) (TIM1 + 0x4C)))


/************** Advanced-control timers (TIM8) *********/
#define TIM8_CR1    (*((volatile dword *) (TIM8 + 0x0)))
#define TIM8_CR2    (*((volatile dword *) (TIM8 + 0x4)))
#define TIM8_SMCR   (*((volatile dword *) (TIM8 + 0x8)))
#define TIM8_DIER   (*((volatile dword *) (TIM8 + 0xC)))
#define TIM8_SR     (*((volatile dword *) (TIM8 + 0x10)))
#define TIM8_EGR    (*((volatile dword *) (TIM8 + 0x14)))
#define TIM8_CCMR1  (*((volatile dword *) (TIM8 + 0x18)))
#define TIM8_CCMR2  (*((volatile dword *) (TIM8 + 0x1C)))
#define TIM8_CCER   (*((volatile dword *) (TIM8 + 0x20)))
#define TIM8_CNT    (*((volatile dword *) (TIM8 + 0x24)))
#define TIM8_PSC    (*((volatile dword *) (TIM8 + 0x28)))
#define TIM8_ARR    (*((volatile dword *) (TIM8 + 0x2C)))
#define TIM8_RCR    (*((volatile dword *) (TIM8 + 0x30)))
#define TIM8_CCR1   (*((volatile dword *) (TIM8 + 0x34)))
#define TIM8_CCR2   (*((volatile dword *) (TIM8 + 0x38)))
#define TIM8_CCR3   (*((volatile dword *) (TIM8 + 0x3C)))
#define TIM8_CCR4   (*((volatile dword *) (TIM8 + 0x40)))
#define TIM8_BDTR   (*((volatile dword *) (TIM8 + 0x44)))
#define TIM8_DCR    (*((volatile dword *) (TIM8 + 0x48)))
#define TIM8_DMAR   (*((volatile dword *) (TIM8 + 0x4C)))



/************** General-purpose timers (TIM2/TIM3/TIM4/TIM5) *********/
#define TIM2_CR1    (*((volatile dword *) (TIM2 + 0x0)))
#define TIM2_CR2    (*((volatile dword *) (TIM2 + 0x4)))
#define TIM2_SMCR   (*((volatile dword *) (TIM2 + 0x8)))
#define TIM2_DIER   (*((volatile dword *) (TIM2 + 0xC)))
#define TIM2_SR     (*((volatile dword *) (TIM2 + 0x10)))
#define TIM2_EGR    (*((volatile dword *) (TIM2 + 0x14)))
#define TIM2_CCMR1  (*((volatile dword *) (TIM2 + 0x18)))
#define TIM2_CCMR2  (*((volatile dword *) (TIM2 + 0x1C)))
#define TIM2_CCER   (*((volatile dword *) (TIM2 + 0x20)))
#define TIM2_CNT    (*((volatile dword *) (TIM2 + 0x24)))
#define TIM2_PSC    (*((volatile dword *) (TIM2 + 0x28)))
#define TIM2_ARR    (*((volatile dword *) (TIM2 + 0x2C)))

#define TIM2_CCR1   (*((volatile dword *) (TIM2 + 0x34)))
#define TIM2_CCR2   (*((volatile dword *) (TIM2 + 0x38)))
#define TIM2_CCR3   (*((volatile dword *) (TIM2 + 0x3C)))
#define TIM2_CCR4   (*((volatile dword *) (TIM2 + 0x40)))

#define TIM2_DCR    (*((volatile dword *) (TIM2 + 0x48)))
#define TIM2_DMAR   (*((volatile dword *) (TIM2 + 0x4C)))


#define TIM3_CR1    (*((volatile dword *) (TIM3 + 0x0)))
#define TIM3_CR2    (*((volatile dword *) (TIM3 + 0x4)))
#define TIM3_SMCR   (*((volatile dword *) (TIM3 + 0x8)))
#define TIM3_DIER   (*((volatile dword *) (TIM3 + 0xC)))
#define TIM3_SR     (*((volatile dword *) (TIM3 + 0x10)))
#define TIM3_EGR    (*((volatile dword *) (TIM3 + 0x14)))
#define TIM3_CCMR1  (*((volatile dword *) (TIM3 + 0x18)))
#define TIM3_CCMR2  (*((volatile dword *) (TIM3 + 0x1C)))
#define TIM3_CCER   (*((volatile dword *) (TIM3 + 0x20)))
#define TIM3_CNT    (*((volatile dword *) (TIM3 + 0x24)))
#define TIM3_PSC    (*((volatile dword *) (TIM3 + 0x28)))
#define TIM3_ARR    (*((volatile dword *) (TIM3 + 0x2C)))

#define TIM3_CCR1   (*((volatile dword *) (TIM3 + 0x34)))
#define TIM3_CCR2   (*((volatile dword *) (TIM3 + 0x38)))
#define TIM3_CCR3   (*((volatile dword *) (TIM3 + 0x3C)))
#define TIM3_CCR4   (*((volatile dword *) (TIM3 + 0x40)))

#define TIM3_DCR    (*((volatile dword *) (TIM3 + 0x48)))
#define TIM3_DMAR   (*((volatile dword *) (TIM3 + 0x4C)))


#define TIM4_CR1    (*((volatile dword *) (TIM4 + 0x0)))
#define TIM4_CR2    (*((volatile dword *) (TIM4 + 0x4)))
#define TIM4_SMCR   (*((volatile dword *) (TIM4 + 0x8)))
#define TIM4_DIER   (*((volatile dword *) (TIM4 + 0xC)))
#define TIM4_SR     (*((volatile dword *) (TIM4 + 0x10)))
#define TIM4_EGR    (*((volatile dword *) (TIM4 + 0x14)))
#define TIM4_CCMR1  (*((volatile dword *) (TIM4 + 0x18)))
#define TIM4_CCMR2  (*((volatile dword *) (TIM4 + 0x1C)))
#define TIM4_CCER   (*((volatile dword *) (TIM4 + 0x20)))
#define TIM4_CNT    (*((volatile dword *) (TIM4 + 0x24)))
#define TIM4_PSC    (*((volatile dword *) (TIM4 + 0x28)))
#define TIM4_ARR    (*((volatile dword *) (TIM4 + 0x2C)))

#define TIM4_CCR1   (*((volatile dword *) (TIM4 + 0x34)))
#define TIM4_CCR2   (*((volatile dword *) (TIM4 + 0x38)))
#define TIM4_CCR3   (*((volatile dword *) (TIM4 + 0x3C)))
#define TIM4_CCR4   (*((volatile dword *) (TIM4 + 0x40)))

#define TIM4_DCR    (*((volatile dword *) (TIM4 + 0x48)))
#define TIM4_DMAR   (*((volatile dword *) (TIM4 + 0x4C)))


#define TIM5_CR1    (*((volatile dword *) (TIM5 + 0x0)))
#define TIM5_CR2    (*((volatile dword *) (TIM5 + 0x4)))
#define TIM5_SMCR   (*((volatile dword *) (TIM5 + 0x8)))
#define TIM5_DIER   (*((volatile dword *) (TIM5 + 0xC)))
#define TIM5_SR     (*((volatile dword *) (TIM5 + 0x10)))
#define TIM5_EGR    (*((volatile dword *) (TIM5 + 0x14)))
#define TIM5_CCMR1  (*((volatile dword *) (TIM5 + 0x18)))
#define TIM5_CCMR2  (*((volatile dword *) (TIM5 + 0x1C)))
#define TIM5_CCER   (*((volatile dword *) (TIM5 + 0x20)))
#define TIM5_CNT    (*((volatile dword *) (TIM5 + 0x24)))
#define TIM5_PSC    (*((volatile dword *) (TIM5 + 0x28)))
#define TIM5_ARR    (*((volatile dword *) (TIM5 + 0x2C)))

#define TIM5_CCR1   (*((volatile dword *) (TIM5 + 0x34)))
#define TIM5_CCR2   (*((volatile dword *) (TIM5 + 0x38)))
#define TIM5_CCR3   (*((volatile dword *) (TIM5 + 0x3C)))
#define TIM5_CCR4   (*((volatile dword *) (TIM5 + 0x40)))

#define TIM5_DCR    (*((volatile dword *) (TIM5 + 0x48)))
#define TIM5_DMAR   (*((volatile dword *) (TIM5 + 0x4C)))



/************** Basic timers (TIM6/TIM7) ******************/
#define TIM6_CR1    (*((volatile dword *) (TIM6 + 0x0)))
#define TIM6_CR2    (*((volatile dword *) (TIM6 + 0x4)))

#define TIM6_DIER   (*((volatile dword *) (TIM6 + 0xC)))
#define TIM6_SR     (*((volatile dword *) (TIM6 + 0x10)))
#define TIM6_EGR    (*((volatile dword *) (TIM6 + 0x14)))

#define TIM6_CNT    (*((volatile dword *) (TIM6 + 0x24)))
#define TIM6_PSC    (*((volatile dword *) (TIM6 + 0x28)))
#define TIM6_ARR    (*((volatile dword *) (TIM6 + 0x2C)))


#define TIM7_CR1    (*((volatile dword *) (TIM7 + 0x0)))
#define TIM7_CR2    (*((volatile dword *) (TIM7 + 0x4)))

#define TIM7_DIER   (*((volatile dword *) (TIM7 + 0xC)))
#define TIM7_SR     (*((volatile dword *) (TIM7 + 0x10)))
#define TIM7_EGR    (*((volatile dword *) (TIM7 + 0x14)))

#define TIM7_CNT    (*((volatile dword *) (TIM7 + 0x24)))
#define TIM7_PSC    (*((volatile dword *) (TIM7 + 0x28)))
#define TIM7_ARR    (*((volatile dword *) (TIM7 + 0x2C)))



/************** RTC register *****************************/
#define RTC_CRH    (*((volatile word *) (RTC + 0x0)))
#define RTC_CRL    (*((volatile word *) (RTC + 0x4)))
#define RTC_PRLH   (*((volatile word *) (RTC + 0x8)))
#define RTC_PRLL   (*((volatile word *) (RTC + 0xC)))
#define RTC_DIVH   (*((volatile word *) (RTC + 0x10)))
#define RTC_DIVL   (*((volatile word *) (RTC + 0x14)))
#define RTC_CNTH   (*((volatile word *) (RTC + 0x18)))
#define RTC_CNTL   (*((volatile word *) (RTC + 0x1C)))
#define RTC_ALRH   (*((volatile word *) (RTC + 0x20)))
#define RTC_ALRL   (*((volatile word *) (RTC + 0x24)))


/************** Independent watchdog (IWDG) *************/
#define IWDG_KR    (*((volatile dword *) (IWDG + 0x0)))
#define IWDG_PR    (*((volatile dword *) (IWDG + 0x4)))
#define IWDG_RLR   (*((volatile dword *) (IWDG + 0x8)))
#define IWDG_SR    (*((volatile dword *) (IWDG + 0xC)))


/************** Window watchdog (WWDG) *****************/
#define WWDG_CR    (*((volatile dword *) (WWDG + 0x0)))
#define WWDG_CFR   (*((volatile dword *) (WWDG + 0x4)))
#define WWDG_SR    (*((volatile dword *) (WWDG + 0x8)))


/************** Flexible static memory controller (FSMC) */
#define FSMC_BCR1  (*((volatile dword *) (FSMC + 0x0)))
#define FSMC_BTR1  (*((volatile dword *) (FSMC + 0x4)))

#define FSMC_BCR2  (*((volatile dword *) (FSMC + 0x8)))
#define FSMC_BTR2  (*((volatile dword *) (FSMC + 0xC)))

#define FSMC_BCR3  (*((volatile dword *) (FSMC + 0x10)))
#define FSMC_BTR3  (*((volatile dword *) (FSMC + 0x14)))

#define FSMC_BCR4  (*((volatile dword *) (FSMC + 0x18)))
#define FSMC_BTR4  (*((volatile dword *) (FSMC + 0x1C)))

#define FSMC_BWTR1 (*((volatile dword *) (FSMC + 0x104)))
#define FSMC_BWTR2 (*((volatile dword *) (FSMC + 0x10C)))
#define FSMC_BWTR3 (*((volatile dword *) (FSMC + 0x114)))
#define FSMC_BWTR4 (*((volatile dword *) (FSMC + 0x11C)))

#define FSMC_PCR2  (*((volatile dword *) (FSMC + 0x60)))
#define FSMC_PCR3  (*((volatile dword *) (FSMC + 0x80)))
#define FSMC_PCR4  (*((volatile dword *) (FSMC + 0xA0)))
#define FSMC_SR2   (*((volatile dword *) (FSMC + 0x64)))
#define FSMC_SR3   (*((volatile dword *) (FSMC + 0x84)))
#define FSMC_SR4   (*((volatile dword *) (FSMC + 0xA4)))

#define FSMC_PMEM2 (*((volatile dword *) (FSMC + 0x68)))
#define FSMC_PMEM3 (*((volatile dword *) (FSMC + 0x88)))
#define FSMC_PMEM4 (*((volatile dword *) (FSMC + 0xA8)))

#define FSMC_PATT2 (*((volatile dword *) (FSMC + 0x6C)))
#define FSMC_PATT3 (*((volatile dword *) (FSMC + 0x8C)))
#define FSMC_PATT4 (*((volatile dword *) (FSMC + 0xAC)))

#define FSMC_PIO4  (*((volatile dword *) (FSMC + 0xB0)))
#define FSMC_ECCR2 (*((volatile dword *) (FSMC + 0x54)))
#define FSMC_ECCR3 (*((volatile dword *) (FSMC + 0x74)))


/************** SDIO Speicherkarten-Interface ***********/
#define SDIO_POWER   (*((volatile dword *) (SDIO +    0)))
#define SDIO_CLKCR   (*((volatile dword *) (SDIO +    4)))
#define SDIO_ARG     (*((volatile dword *) (SDIO +    8)))
#define SDIO_CMD     (*((volatile dword *) (SDIO + 0x0C)))
#define SDIO_RESPCMD (*((volatile dword *) (SDIO + 0x10)))
#define SDIO_RESP1   (*((volatile dword *) (SDIO + 0x14)))
#define SDIO_RESP2   (*((volatile dword *) (SDIO + 0x18)))
#define SDIO_RESP3   (*((volatile dword *) (SDIO + 0x1C)))
#define SDIO_RESP4   (*((volatile dword *) (SDIO + 0x20)))
#define SDIO_DTIMER  (*((volatile dword *) (SDIO + 0x24)))
#define SDIO_DLEN    (*((volatile dword *) (SDIO + 0x28)))
#define SDIO_DCTRL   (*((volatile dword *) (SDIO + 0x2C)))
#define SDIO_DCOUNT  (*((volatile dword *) (SDIO + 0x30)))
#define SDIO_STA     (*((volatile dword *) (SDIO + 0x34)))
#define SDIO_ICR     (*((volatile dword *) (SDIO + 0x38)))
#define SDIO_MASK    (*((volatile dword *) (SDIO + 0x3C)))
#define SDIO_FIFOCNT (*((volatile dword *) (SDIO + 0x48)))
#define SDIO_FIFO    (*((volatile dword *) (SDIO + 0x80)))


/************** USB device interface (USB) **************/
#define USB_EpRegs(x) (*(volatile dword *)(USB + 4*(x)))
#define USB_EP0R    (*((volatile dword *) (USB + 0x0)))
#define USB_EP1R    (*((volatile dword *) (USB + 0x4)))
#define USB_EP2R    (*((volatile dword *) (USB + 0x8)))
#define USB_EP3R    (*((volatile dword *) (USB + 0xC)))
#define USB_EP4R    (*((volatile dword *) (USB + 0x10)))
#define USB_EP5R    (*((volatile dword *) (USB + 0x14)))
#define USB_EP6R    (*((volatile dword *) (USB + 0x18)))
#define USB_EP7R    (*((volatile dword *) (USB + 0x1C)))
#define USB_CNTR    (*((volatile dword *) (USB + 0x40)))
#define USB_ISTR    (*((volatile dword *) (USB + 0x44)))
#define USB_FNR     (*((volatile dword *) (USB + 0x48)))
#define USB_DADDR   (*((volatile dword *) (USB + 0x4C)))
#define USB_BTABLE  (*((volatile dword *) (USB + 0x50)))


/************** Controller area network (bxCAN) ********/
#define CAN1_MCR      (*((volatile dword *) (CAN1 + 0x0)))
#define CAN1_MSR      (*((volatile dword *) (CAN1 + 0x4)))
#define CAN1_TSR      (*((volatile dword *) (CAN1 + 0x8)))
#define CAN1_RF0R     (*((volatile dword *) (CAN1 + 0xC)))
#define CAN1_RF1R     (*((volatile dword *) (CAN1 + 0x10)))
#define CAN1_IER      (*((volatile dword *) (CAN1 + 0x14)))
#define CAN1_ESR      (*((volatile dword *) (CAN1 + 0x18)))
#define CAN1_BTR      (*((volatile dword *) (CAN1 + 0x1C)))

#define CAN1_TI0R     (*((volatile dword *) (CAN1 + 0x180)))
#define CAN1_TDT0R    (*((volatile dword *) (CAN1 + 0x184)))
#define CAN1_TDL0R    (*((volatile dword *) (CAN1 + 0x188)))
#define CAN1_TDH0R    (*((volatile dword *) (CAN1 + 0x18C)))
                                                
#define CAN1_TI1R     (*((volatile dword *) (CAN1 + 0x190)))
#define CAN1_TDT1R    (*((volatile dword *) (CAN1 + 0x194)))
#define CAN1_TDL1R    (*((volatile dword *) (CAN1 + 0x198)))
#define CAN1_TDH1R    (*((volatile dword *) (CAN1 + 0x19C)))

#define CAN1_TI2R     (*((volatile dword *) (CAN1 + 0x1A0)))
#define CAN1_TDT2R    (*((volatile dword *) (CAN1 + 0x1A4)))
#define CAN1_TDL2R    (*((volatile dword *) (CAN1 + 0x1A8)))
#define CAN1_TDH2R    (*((volatile dword *) (CAN1 + 0x1AC)))

#define CAN1_RI0R     (*((volatile dword *) (CAN1 + 0x1B0)))
#define CAN1_RDT0R    (*((volatile dword *) (CAN1 + 0x1B4)))
#define CAN1_RDL0R    (*((volatile dword *) (CAN1 + 0x1B8)))
#define CAN1_RDH0R    (*((volatile dword *) (CAN1 + 0x1BC)))

#define CAN1_RI1R     (*((volatile dword *) (CAN1 + 0x1C0)))
#define CAN1_RDT1R    (*((volatile dword *) (CAN1 + 0x1C4)))
#define CAN1_RDL1R    (*((volatile dword *) (CAN1 + 0x1C8)))
#define CAN1_RDH1R    (*((volatile dword *) (CAN1 + 0x1CC)))

#define CAN1_FMR      (*((volatile dword *) (CAN1 + 0x200)))
#define CAN1_FM1R     (*((volatile dword *) (CAN1 + 0x204)))
#define CAN1_FS1R     (*((volatile dword *) (CAN1 + 0x20C)))
#define CAN1_FFA1R    (*((volatile dword *) (CAN1 + 0x214)))
#define CAN1_FA1R     (*((volatile dword *) (CAN1 + 0x21C)))

#define CAN1_F0R1     (*((volatile dword *) (CAN1 + 0x240)))
#define CAN1_F0R2     (*((volatile dword *) (CAN1 + 0x244)))
#define CAN1_F1R1     (*((volatile dword *) (CAN1 + 0x248)))
#define CAN1_F1R2     (*((volatile dword *) (CAN1 + 0x24C)))
#define CAN1_F2R1     (*((volatile dword *) (CAN1 + 0x250)))
#define CAN1_F2R2     (*((volatile dword *) (CAN1 + 0x254)))
#define CAN1_F3R1     (*((volatile dword *) (CAN1 + 0x258)))
#define CAN1_F3R2     (*((volatile dword *) (CAN1 + 0x25C)))
#define CAN1_F4R1     (*((volatile dword *) (CAN1 + 0x260)))
#define CAN1_F4R2     (*((volatile dword *) (CAN1 + 0x264)))
#define CAN1_F5R1     (*((volatile dword *) (CAN1 + 0x268)))
#define CAN1_F5R2     (*((volatile dword *) (CAN1 + 0x26C)))
#define CAN1_F6R1     (*((volatile dword *) (CAN1 + 0x270)))
#define CAN1_F6R2     (*((volatile dword *) (CAN1 + 0x274)))
#define CAN1_F7R1     (*((volatile dword *) (CAN1 + 0x278)))
#define CAN1_F7R2     (*((volatile dword *) (CAN1 + 0x27C)))
#define CAN1_F8R1     (*((volatile dword *) (CAN1 + 0x280)))
#define CAN1_F8R2     (*((volatile dword *) (CAN1 + 0x284)))
#define CAN1_F9R1     (*((volatile dword *) (CAN1 + 0x288)))
#define CAN1_F9R2     (*((volatile dword *) (CAN1 + 0x28C)))
#define CAN1_F10R1    (*((volatile dword *) (CAN1 + 0x290)))
#define CAN1_F10R2    (*((volatile dword *) (CAN1 + 0x294)))
#define CAN1_F11R1    (*((volatile dword *) (CAN1 + 0x298)))
#define CAN1_F11R2    (*((volatile dword *) (CAN1 + 0x29C)))
#define CAN1_F12R1    (*((volatile dword *) (CAN1 + 0x2A0)))
#define CAN1_F12R2    (*((volatile dword *) (CAN1 + 0x2A4)))
#define CAN1_F13R1    (*((volatile dword *) (CAN1 + 0x2A8)))
#define CAN1_F13R2    (*((volatile dword *) (CAN1 + 0x2AC)))
#define CAN1_F14R1    (*((volatile dword *) (CAN1 + 0x2B0)))
#define CAN1_F14R2    (*((volatile dword *) (CAN1 + 0x2B4)))
#define CAN1_F15R1    (*((volatile dword *) (CAN1 + 0x2B8)))
#define CAN1_F15R2    (*((volatile dword *) (CAN1 + 0x2BC)))
#define CAN1_F16R1    (*((volatile dword *) (CAN1 + 0x2C0)))
#define CAN1_F16R2    (*((volatile dword *) (CAN1 + 0x2C4)))
#define CAN1_F17R1    (*((volatile dword *) (CAN1 + 0x2C8)))
#define CAN1_F17R2    (*((volatile dword *) (CAN1 + 0x2CC)))
#define CAN1_F18R1    (*((volatile dword *) (CAN1 + 0x2D0)))
#define CAN1_F18R2    (*((volatile dword *) (CAN1 + 0x2D4)))
#define CAN1_F19R1    (*((volatile dword *) (CAN1 + 0x2D8)))
#define CAN1_F19R2    (*((volatile dword *) (CAN1 + 0x2DC)))
#define CAN1_F20R1    (*((volatile dword *) (CAN1 + 0x2E0)))
#define CAN1_F20R2    (*((volatile dword *) (CAN1 + 0x2E4)))
#define CAN1_F21R1    (*((volatile dword *) (CAN1 + 0x2E8)))
#define CAN1_F21R2    (*((volatile dword *) (CAN1 + 0x2EC)))
#define CAN1_F22R1    (*((volatile dword *) (CAN1 + 0x2F0)))
#define CAN1_F22R2    (*((volatile dword *) (CAN1 + 0x2F4)))
#define CAN1_F23R1    (*((volatile dword *) (CAN1 + 0x2F8)))
#define CAN1_F23R2    (*((volatile dword *) (CAN1 + 0x2FC)))
#define CAN1_F24R1    (*((volatile dword *) (CAN1 + 0x200)))
#define CAN1_F24R2    (*((volatile dword *) (CAN1 + 0x204)))
#define CAN1_F25R1    (*((volatile dword *) (CAN1 + 0x208)))
#define CAN1_F25R2    (*((volatile dword *) (CAN1 + 0x20C)))
#define CAN1_F26R1    (*((volatile dword *) (CAN1 + 0x210)))
#define CAN1_F26R2    (*((volatile dword *) (CAN1 + 0x214)))
#define CAN1_F27R1    (*((volatile dword *) (CAN1 + 0x218)))
#define CAN1_F27R2    (*((volatile dword *) (CAN1 + 0x21C)))

/**********/
#define CAN2_MCR      (*((volatile dword *) (CAN2 + 0x0)))
#define CAN2_MSR      (*((volatile dword *) (CAN2 + 0x4)))
#define CAN2_TSR      (*((volatile dword *) (CAN2 + 0x8)))
#define CAN2_RF0R     (*((volatile dword *) (CAN2 + 0xC)))
#define CAN2_RF1R     (*((volatile dword *) (CAN2 + 0x10)))
#define CAN2_IER      (*((volatile dword *) (CAN2 + 0x14)))
#define CAN2_ESR      (*((volatile dword *) (CAN2 + 0x18)))
#define CAN2_BTR      (*((volatile dword *) (CAN2 + 0x1C)))

#define CAN2_TI0R     (*((volatile dword *) (CAN2 + 0x180)))
#define CAN2_TDT0R    (*((volatile dword *) (CAN2 + 0x184)))
#define CAN2_TDL0R    (*((volatile dword *) (CAN2 + 0x188)))
#define CAN2_TDH0R    (*((volatile dword *) (CAN2 + 0x18C)))

#define CAN2_TI1R     (*((volatile dword *) (CAN2 + 0x190)))
#define CAN2_TDT1R    (*((volatile dword *) (CAN2 + 0x194)))
#define CAN2_TDL1R    (*((volatile dword *) (CAN2 + 0x198)))
#define CAN2_TDH1R    (*((volatile dword *) (CAN2 + 0x19C)))

#define CAN2_TI2R     (*((volatile dword *) (CAN2 + 0x1A0)))
#define CAN2_TDT2R    (*((volatile dword *) (CAN2 + 0x1A4)))
#define CAN2_TDL2R    (*((volatile dword *) (CAN2 + 0x1A8)))
#define CAN2_TDH2R    (*((volatile dword *) (CAN2 + 0x1AC)))

#define CAN2_RI0R     (*((volatile dword *) (CAN2 + 0x1B0)))
#define CAN2_RDT0R    (*((volatile dword *) (CAN2 + 0x1B4)))
#define CAN2_RDL0R    (*((volatile dword *) (CAN2 + 0x1B8)))
#define CAN2_RDH0R    (*((volatile dword *) (CAN2 + 0x1BC)))

#define CAN2_RI1R     (*((volatile dword *) (CAN2 + 0x1C0)))
#define CAN2_RDT1R    (*((volatile dword *) (CAN2 + 0x1C4)))
#define CAN2_RDL1R    (*((volatile dword *) (CAN2 + 0x1C8)))
#define CAN2_RDH1R    (*((volatile dword *) (CAN2 + 0x1CC)))

#define CAN2_FMR      (*((volatile dword *) (CAN2 + 0x200)))
#define CAN2_FM1R     (*((volatile dword *) (CAN2 + 0x204)))
#define CAN2_FS1R     (*((volatile dword *) (CAN2 + 0x20C)))
#define CAN2_FFA1R    (*((volatile dword *) (CAN2 + 0x214)))
#define CAN2_FA1R     (*((volatile dword *) (CAN2 + 0x21C)))

#define CAN2_F0R1     (*((volatile dword *) (CAN2 + 0x240)))
#define CAN2_F0R2     (*((volatile dword *) (CAN2 + 0x244)))
#define CAN2_F1R1     (*((volatile dword *) (CAN2 + 0x248)))
#define CAN2_F1R2     (*((volatile dword *) (CAN2 + 0x24C)))
#define CAN2_F2R1     (*((volatile dword *) (CAN2 + 0x250)))
#define CAN2_F2R2     (*((volatile dword *) (CAN2 + 0x254)))
#define CAN2_F3R1     (*((volatile dword *) (CAN2 + 0x258)))
#define CAN2_F3R2     (*((volatile dword *) (CAN2 + 0x25C)))
#define CAN2_F4R1     (*((volatile dword *) (CAN2 + 0x260)))
#define CAN2_F4R2     (*((volatile dword *) (CAN2 + 0x264)))
#define CAN2_F5R1     (*((volatile dword *) (CAN2 + 0x268)))
#define CAN2_F5R2     (*((volatile dword *) (CAN2 + 0x26C)))
#define CAN2_F6R1     (*((volatile dword *) (CAN2 + 0x270)))
#define CAN2_F6R2     (*((volatile dword *) (CAN2 + 0x274)))
#define CAN2_F7R1     (*((volatile dword *) (CAN2 + 0x278)))
#define CAN2_F7R2     (*((volatile dword *) (CAN2 + 0x27C)))
#define CAN2_F8R1     (*((volatile dword *) (CAN2 + 0x280)))
#define CAN2_F8R2     (*((volatile dword *) (CAN2 + 0x284)))
#define CAN2_F9R1     (*((volatile dword *) (CAN2 + 0x288)))
#define CAN2_F9R2     (*((volatile dword *) (CAN2 + 0x28C)))
#define CAN2_F10R1    (*((volatile dword *) (CAN2 + 0x290)))
#define CAN2_F10R2    (*((volatile dword *) (CAN2 + 0x294)))
#define CAN2_F11R1    (*((volatile dword *) (CAN2 + 0x298)))
#define CAN2_F11R2    (*((volatile dword *) (CAN2 + 0x29C)))
#define CAN2_F12R1    (*((volatile dword *) (CAN2 + 0x2A0)))
#define CAN2_F12R2    (*((volatile dword *) (CAN2 + 0x2A4)))
#define CAN2_F13R1    (*((volatile dword *) (CAN2 + 0x2A8)))
#define CAN2_F13R2    (*((volatile dword *) (CAN2 + 0x2AC)))
#define CAN2_F14R1    (*((volatile dword *) (CAN2 + 0x2B0)))
#define CAN2_F14R2    (*((volatile dword *) (CAN2 + 0x2B4)))
#define CAN2_F15R1    (*((volatile dword *) (CAN2 + 0x2B8)))
#define CAN2_F15R2    (*((volatile dword *) (CAN2 + 0x2BC)))
#define CAN2_F16R1    (*((volatile dword *) (CAN2 + 0x2C0)))
#define CAN2_F16R2    (*((volatile dword *) (CAN2 + 0x2C4)))
#define CAN2_F17R1    (*((volatile dword *) (CAN2 + 0x2C8)))
#define CAN2_F17R2    (*((volatile dword *) (CAN2 + 0x2CC)))
#define CAN2_F18R1    (*((volatile dword *) (CAN2 + 0x2D0)))
#define CAN2_F18R2    (*((volatile dword *) (CAN2 + 0x2D4)))
#define CAN2_F19R1    (*((volatile dword *) (CAN2 + 0x2D8)))
#define CAN2_F19R2    (*((volatile dword *) (CAN2 + 0x2DC)))
#define CAN2_F20R1    (*((volatile dword *) (CAN2 + 0x2E0)))
#define CAN2_F20R2    (*((volatile dword *) (CAN2 + 0x2E4)))
#define CAN2_F21R1    (*((volatile dword *) (CAN2 + 0x2E8)))
#define CAN2_F21R2    (*((volatile dword *) (CAN2 + 0x2EC)))
#define CAN2_F22R1    (*((volatile dword *) (CAN2 + 0x2F0)))
#define CAN2_F22R2    (*((volatile dword *) (CAN2 + 0x2F4)))
#define CAN2_F23R1    (*((volatile dword *) (CAN2 + 0x2F8)))
#define CAN2_F23R2    (*((volatile dword *) (CAN2 + 0x2FC)))
#define CAN2_F24R1    (*((volatile dword *) (CAN2 + 0x200)))
#define CAN2_F24R2    (*((volatile dword *) (CAN2 + 0x204)))
#define CAN2_F25R1    (*((volatile dword *) (CAN2 + 0x208)))
#define CAN2_F25R2    (*((volatile dword *) (CAN2 + 0x20C)))
#define CAN2_F26R1    (*((volatile dword *) (CAN2 + 0x210)))
#define CAN2_F26R2    (*((volatile dword *) (CAN2 + 0x214)))
#define CAN2_F27R1    (*((volatile dword *) (CAN2 + 0x218)))
#define CAN2_F27R2    (*((volatile dword *) (CAN2 + 0x21C)))


/************** Serial peripheral interface (SPI) **********/
#define SPI1_CR1     (*((volatile dword *) (SPI1 + 0x0)))
#define SPI1_CR2     (*((volatile dword *) (SPI1 + 0x4)))
#define SPI1_SR      (*((volatile dword *) (SPI1 + 0x8)))
#define SPI1_DR      (*((volatile dword *) (SPI1 + 0xC)))
#define SPI1_CRCPR   (*((volatile dword *) (SPI1 + 0x10)))
#define SPI1_RXCRCR  (*((volatile dword *) (SPI1 + 0x14)))
#define SPI1_TXCRCR  (*((volatile dword *) (SPI1 + 0x18)))
#define SPI1_I2SCFGR (*((volatile dword *) (SPI1 + 0x1C)))
#define SPI1_I2SPR   (*((volatile dword *) (SPI1 + 0x20)))

#define SPI2_CR1     (*((volatile dword *) (SPI2 + 0x0)))
#define SPI2_CR2     (*((volatile dword *) (SPI2 + 0x4)))
#define SPI2_SR      (*((volatile dword *) (SPI2 + 0x8)))
#define SPI2_DR      (*((volatile dword *) (SPI2 + 0xC)))
#define SPI2_CRCPR   (*((volatile dword *) (SPI2 + 0x10)))
#define SPI2_RXCRCR  (*((volatile dword *) (SPI2 + 0x14)))
#define SPI2_TXCRCR  (*((volatile dword *) (SPI2 + 0x18)))
#define SPI2_I2SCFGR (*((volatile dword *) (SPI2 + 0x1C)))
#define SPI2_I2SPR   (*((volatile dword *) (SPI2 + 0x20)))

#define SPI3_CR1     (*((volatile dword *) (SPI3 + 0x0)))
#define SPI3_CR2     (*((volatile dword *) (SPI3 + 0x4)))
#define SPI3_SR      (*((volatile dword *) (SPI3 + 0x8)))
#define SPI3_DR      (*((volatile dword *) (SPI3 + 0xC)))
#define SPI3_CRCPR   (*((volatile dword *) (SPI3 + 0x10)))
#define SPI3_RXCRCR  (*((volatile dword *) (SPI3 + 0x14)))
#define SPI3_TXCRCR  (*((volatile dword *) (SPI3 + 0x18)))
#define SPI3_I2SCFGR (*((volatile dword *) (SPI3 + 0x1C)))
#define SPI3_I2SPR   (*((volatile dword *) (SPI3 + 0x20)))


/************** I2C *************************************/
/* I2C1 */
#define I2C1_CR1      (*((volatile dword *) (I2C1 + 0x0)))
#define I2C1_CR2      (*((volatile dword *) (I2C1 + 0x4)))
#define I2C1_OAR1     (*((volatile dword *) (I2C1 + 0x8)))
#define I2C1_OAR2     (*((volatile dword *) (I2C1 + 0xC)))
#define I2C1_DR       (*((volatile dword *) (I2C1 + 0x10)))
#define I2C1_SR1      (*((volatile dword *) (I2C1 + 0x14)))
#define I2C1_SR2      (*((volatile dword *) (I2C1 + 0x18)))
#define I2C1_CCR      (*((volatile dword *) (I2C1 + 0x1C)))
#define I2C1_TRISE    (*((volatile dword *) (I2C1 + 0x20)))

/* I2C2 */
#define I2C2_CR1      (*((volatile dword *) (I2C2 + 0x0)))
#define I2C2_CR2      (*((volatile dword *) (I2C2 + 0x4)))
#define I2C2_OAR1     (*((volatile dword *) (I2C2 + 0x8)))
#define I2C2_OAR2     (*((volatile dword *) (I2C2 + 0xC)))
#define I2C2_DR       (*((volatile dword *) (I2C2 + 0x10)))
#define I2C2_SR1      (*((volatile dword *) (I2C2 + 0x14)))
#define I2C2_SR2      (*((volatile dword *) (I2C2 + 0x18)))
#define I2C2_CCR      (*((volatile dword *) (I2C2 + 0x1C)))
#define I2C2_TRISE    (*((volatile dword *) (I2C2 + 0x20)))


/************** U(S)ART ***********************************/
#define USART1_ISR   (*((volatile dword *) (USART1 + 0x0)))   // im RefMan: USARTx_SR
#define USART1_RDR   (*((volatile dword *) (USART1 + 0x4)))   // im RefMan: USARTx_DR
#define USART1_TDR   (*((volatile dword *) (USART1 + 0x4)))   // im RefMan: USARTx_DR
#define USART1_BRR   (*((volatile dword *) (USART1 + 0x8)))
#define USART1_CR1   (*((volatile dword *) (USART1 + 0xC)))
#define USART1_CR2   (*((volatile dword *) (USART1 + 0x10)))
#define USART1_CR3   (*((volatile dword *) (USART1 + 0x14)))
#define USART1_GTPR  (*((volatile dword *) (USART1 + 0x18)))

#define USART2_ISR   (*((volatile dword *) (USART2 + 0x0)))   // im RefMan: USARTx_SR
#define USART2_TDR   (*((volatile dword *) (USART2 + 0x4)))   // im RefMan: USARTx_DR
#define USART2_RDR   (*((volatile dword *) (USART2 + 0x4)))   // im RefMan: USARTx_DR
#define USART2_BRR   (*((volatile dword *) (USART2 + 0x8)))
#define USART2_CR1   (*((volatile dword *) (USART2 + 0xC)))
#define USART2_CR2   (*((volatile dword *) (USART2 + 0x10)))
#define USART2_CR3   (*((volatile dword *) (USART2 + 0x14)))
#define USART2_GTPR  (*((volatile dword *) (USART2 + 0x18)))

#define USART3_ISR   (*((volatile dword *) (USART3 + 0x0)))   // im RefMan: USARTx_SR
#define USART3_TDR   (*((volatile dword *) (USART3 + 0x4)))   // im RefMan: USARTx_DR
#define USART3_RDR   (*((volatile dword *) (USART3 + 0x4)))   // im RefMan: USARTx_DR
#define USART3_BRR   (*((volatile dword *) (USART3 + 0x8)))
#define USART3_CR1   (*((volatile dword *) (USART3 + 0xC)))
#define USART3_CR2   (*((volatile dword *) (USART3 + 0x10)))
#define USART3_CR3   (*((volatile dword *) (USART3 + 0x14)))
#define USART3_GTPR  (*((volatile dword *) (USART3 + 0x18)))

#define UART4_ISR   (*((volatile dword *) (UART4 + 0x0)))   // im RefMan: USARTx_SR
#define UART4_TDR   (*((volatile dword *) (UART4 + 0x4)))   // im RefMan: USARTx_DR
#define UART4_RDR   (*((volatile dword *) (UART4 + 0x4)))   // im RefMan: USARTx_DR
#define UART4_BRR   (*((volatile dword *) (UART4 + 0x8)))
#define UART4_CR1   (*((volatile dword *) (UART4 + 0xC)))
#define UART4_CR2   (*((volatile dword *) (UART4 + 0x10)))
#define UART4_CR3   (*((volatile dword *) (UART4 + 0x14)))
#define UART4_GTPR  (*((volatile dword *) (UART4 + 0x18)))

#define UART5_ISR   (*((volatile dword *) (UART5 + 0x0)))   // im RefMan: USARTx_SR
#define UART5_TDR   (*((volatile dword *) (UART5 + 0x4)))   // im RefMan: USARTx_DR
#define UART5_RDR   (*((volatile dword *) (UART5 + 0x4)))   // im RefMan: USARTx_DR
#define UART5_BRR   (*((volatile dword *) (UART5 + 0x8)))
#define UART5_CR1   (*((volatile dword *) (UART5 + 0xC)))
#define UART5_CR2   (*((volatile dword *) (UART5 + 0x10)))
#define UART5_CR3   (*((volatile dword *) (UART5 + 0x14)))
#define UART5_GTPR  (*((volatile dword *) (UART5 + 0x18)))


/************** USB on-the-go full-speed (OTG_FS) ***********/


/************** Ethernet (ETH) ******************************/


/************** Device electronic signature *****************/
#define F_SIZE    (*((volatile dword *) 0x1FFFF7E0))
#define U_IDL     (*((volatile dword *) 0x1FFFF7E8))
#define U_IDM     (*((volatile dword *) 0x1FFFF7EC))
#define U_IDH     (*((volatile dword *) 0x1FFFF7F0))


/************** DBG Debug register map **********************/
#define DBGMCU_IDCODE (*((volatile dword *) 0xE0042000))
#define DBGMCU_CR     (*((volatile dword *) 0xE0042004))


/************** Interner Flash - register map **********************/
#define FLASH_ACR     (*((volatile dword *) 0x40022000)) // 
#define FLASH_KEYR    (*((volatile dword *) 0x40022004)) // 
#define FLASH_OPTKEYR (*((volatile dword *) 0x40022008)) //
#define FLASH_SR      (*((volatile dword *) 0x4002200C)) // 
#define FLASH_CR      (*((volatile dword *) 0x40022010)) // 
#define FLASH_AR      (*((volatile dword *) 0x40022014)) // 
#define FLASH_OBR     (*((volatile dword *) 0x4002201C)) // 
#define FLASH_WRPR    (*((volatile dword *) 0x40022020)) // 



/********** Cortex M3 Register **************************************************************************************/

/* System control block */
#define ACTLR (*((volatile dword *) 0xE000E008)) // RW Privileged 0x00000000 Auxiliary Control Register
#define CPUID (*((volatile dword *) 0xE000ED00)) // RO Privileged 0x412FC230 CPUID Base Register on page 4-12
#define ICSR  (*((volatile dword *) 0xE000ED04)) // RW Privileged 0x00000000 Interrupt Control and State Register on page 4-13
#define VTOR  (*((volatile dword *) 0xE000ED08)) // RW Privileged 0x00000000 Vector Table Offset Register on page 4-16 
#define AIRCR (*((volatile dword *) 0xE000ED0C)) // RW Privileged 0xFA050000 Application Interrupt and Reset Control Register on page 4-16
#define SCR   (*((volatile dword *) 0xE000ED10)) // RW Privileged 0x00000000 System Control Register on page 4-19
#define CCR   (*((volatile dword *) 0xE000ED14)) // RW Privileged 0x00000200 Configuration and Control Register on page 4-19
#define SHPR1 (*((volatile dword *) 0xE000ED18)) // RW Privileged 0x00000000 System Handler Priority Register 1 on page 4-21
#define SHPR2 (*((volatile dword *) 0xE000ED1C)) // RW Privileged 0x00000000 System Handler Priority Register 2 on page 4-22
#define SHPR3 (*((volatile dword *) 0xE000ED20)) // RW Privileged 0x00000000 System Handler Priority Register 3 on page 4-22
#define SHCRS (*((volatile dword *) 0xE000ED24)) // RW Privileged 0x00000000 System Handler Control and State Register on page 4-23 
#define CFSR  (*((volatile dword *) 0xE000ED28)) // RW Privileged 0x00000000 Configurable Fault Status Register on page 4-24
#define MMSR  (*((volatile dword *) 0xE000ED28)) // RW Privileged 0x00 MemManage Fault Status Register on page 4-25
#define BFSR  (*((volatile dword *) 0xE000ED29)) // RW Privileged 0x00 BusFault Status Register on page 4-26 
#define UFSR  (*((volatile dword *) 0xE000ED2A)) // RW Privileged 0x0000 UsageFault Status Register on page 4-28 
#define HFSR  (*((volatile dword *) 0xE000ED2C)) // RW Privileged 0x00000000 HardFault Status Register on page 4-30
#define MMAR  (*((volatile dword *) 0xE000ED34)) // RW Privileged Unknown MemManage Fault Address Register on page 4-30
#define BFAR  (*((volatile dword *) 0xE000ED38)) // RW Privileged Unknown BusFault Address Register on page 4-31
#define AFSR  (*((volatile dword *) 0xE000ED3C)) // RW Privileged 0x00000000 Auxiliary Fault Status Register on page 4-31


/* System timer, SysTick */
#define SYST_CSR    (*((volatile dword *) 0xE000E010)) // STCSR RW 0x00000000 SysTick Control and Status Register
#define SYST_RVR    (*((volatile dword *) 0xE000E014)) // STRVR RW Unknown SysTick Reload Value Register
#define SYST_CVR    (*((volatile dword *) 0xE000E018)) // STCVR RW clear Unknown SysTick Current Value Register
#define SYST_CALIB  (*((volatile dword *) 0xE000E01C)) // STCR  RO STCALIB SysTick Calibration Value Register


/* Optional Memory Protection Unit */
#define MPU_TYPE     (*((volatile dword *) 0xE000ED90)) // MPU_TYPE RO 0x00000800 MPU Type Register
#define MPU_CTRL     (*((volatile dword *) 0xE000ED94)) // MPU_CTRL RW 0x00000000 MPU Control Register
#define MPU_RNR      (*((volatile dword *) 0xE000ED98)) // MPU_RNR  RW 0x00000000 MPU Region Number Register
#define MPU_RBAR     (*((volatile dword *) 0xE000ED9C)) // MPU_RBAR RW 0x00000000 MPU Region Base Address Register
#define MPU_RASR     (*((volatile dword *) 0xE000EDA0)) // MPU_RASR RW 0x00000000 MPU Region Attribute and Size Register
#define MPU_RBAR_A1  (*((volatile dword *) 0xE000EDA4)) // MPU_RBAR_A1 0x00000000 MPU alias registers
#define MPU_RASR_A1  (*((volatile dword *) 0xE000EDA8)) // MPU_RASR_A1 0x00000000
#define MPU_RBAR_A2  (*((volatile dword *) 0xE000EDAC)) // MPU_RBAR_A2 0x00000000
#define MPU_RASR_A2  (*((volatile dword *) 0xE000EDB0)) // MPU_RASR_A2 0x00000000
#define MPU_RBAR_A3  (*((volatile dword *) 0xE000EDB4)) // MPU_RBAR_A3 0x00000000
#define MPU_RASR_A3  (*((volatile dword *) 0xE000EDB8)) // MPU_RASR_A3 0x00000000


/* NVIC  Nested  Interrupt Controller */
#define NVIC_STIR   (*((volatile dword *) 0xE000EF00)) // STIR WO 0x00000000 Software Triggered Interrupt Register

#define NVIC_ISER0  (*((volatile dword *) 0xE000E100)) // NVIC_ISER RW 0x00000000 Interrupt Set-Enable Register.
#define NVIC_ISER1  (*((volatile dword *) 0xE000E104)) // NVIC_ISER RW 0x00000000 Interrupt Set-Enable Register.
#define NVIC_ISER2  (*((volatile dword *) 0xE000E108)) // NVIC_ISER RW 0x00000000 Interrupt Set-Enable Register.
#define NVIC_ISER3  (*((volatile dword *) 0xE000E10C)) // NVIC_ISER RW 0x00000000 Interrupt Set-Enable Register.
#define NVIC_ISER4  (*((volatile dword *) 0xE000E110)) // NVIC_ISER RW 0x00000000 Interrupt Set-Enable Register.
#define NVIC_ISER5  (*((volatile dword *) 0xE000E114)) // NVIC_ISER RW 0x00000000 Interrupt Set-Enable Register.
#define NVIC_ISER6  (*((volatile dword *) 0xE000E118)) // NVIC_ISER RW 0x00000000 Interrupt Set-Enable Register.
#define NVIC_ISER7  (*((volatile dword *) 0xE000E11C)) // NVIC_ISER RW 0x00000000 Interrupt Set-Enable Register.

#define NVIC_ICER0  (*((volatile dword *) 0xE000E180)) // NVIC_ICER RW 0x00000000 Interrupt Clear-Enable Register on page 4-4.
#define NVIC_ICER1  (*((volatile dword *) 0xE000E184)) // NVIC_ICER RW 0x00000000 Interrupt Clear-Enable Register on page 4-4.
#define NVIC_ICER2  (*((volatile dword *) 0xE000E188)) // NVIC_ICER RW 0x00000000 Interrupt Clear-Enable Register on page 4-4.
#define NVIC_ICER3  (*((volatile dword *) 0xE000E18C)) // NVIC_ICER RW 0x00000000 Interrupt Clear-Enable Register on page 4-4.
#define NVIC_ICER4  (*((volatile dword *) 0xE000E190)) // NVIC_ICER RW 0x00000000 Interrupt Clear-Enable Register on page 4-4.
#define NVIC_ICER5  (*((volatile dword *) 0xE000E194)) // NVIC_ICER RW 0x00000000 Interrupt Clear-Enable Register on page 4-4.
#define NVIC_ICER6  (*((volatile dword *) 0xE000E198)) // NVIC_ICER RW 0x00000000 Interrupt Clear-Enable Register on page 4-4.
#define NVIC_ICER7  (*((volatile dword *) 0xE000E19C)) // NVIC_ICER RW 0x00000000 Interrupt Clear-Enable Register on page 4-4.

#define NVIC_ISPR0  (*((volatile dword *) 0xE000E200)) // NVIC_ISPR RW 0x00000000 Interrupt Set-Pending Register on page 4-4.
#define NVIC_ISPR1  (*((volatile dword *) 0xE000E204)) // NVIC_ISPR RW 0x00000000 Interrupt Set-Pending Register on page 4-4.
#define NVIC_ISPR2  (*((volatile dword *) 0xE000E208)) // NVIC_ISPR RW 0x00000000 Interrupt Set-Pending Register on page 4-4.
#define NVIC_ISPR3  (*((volatile dword *) 0xE000E20C)) // NVIC_ISPR RW 0x00000000 Interrupt Set-Pending Register on page 4-4.
#define NVIC_ISPR4  (*((volatile dword *) 0xE000E210)) // NVIC_ISPR RW 0x00000000 Interrupt Set-Pending Register on page 4-4.
#define NVIC_ISPR5  (*((volatile dword *) 0xE000E214)) // NVIC_ISPR RW 0x00000000 Interrupt Set-Pending Register on page 4-4.
#define NVIC_ISPR6  (*((volatile dword *) 0xE000E218)) // NVIC_ISPR RW 0x00000000 Interrupt Set-Pending Register on page 4-4.
#define NVIC_ISPR7  (*((volatile dword *) 0xE000E21C)) // NVIC_ISPR RW 0x00000000 Interrupt Set-Pending Register on page 4-4.

#define NVIC_ICPR0  (*((volatile dword *) 0xE000E280)) // NVIC_ICPR RW 0x00000000 Interrupt Clear-Pending Register on page 4-4.
#define NVIC_ICPR1  (*((volatile dword *) 0xE000E284)) // NVIC_ICPR RW 0x00000000 Interrupt Clear-Pending Register on page 4-4.
#define NVIC_ICPR2  (*((volatile dword *) 0xE000E288)) // NVIC_ICPR RW 0x00000000 Interrupt Clear-Pending Register on page 4-4.
#define NVIC_ICPR3  (*((volatile dword *) 0xE000E28C)) // NVIC_ICPR RW 0x00000000 Interrupt Clear-Pending Register on page 4-4.
#define NVIC_ICPR4  (*((volatile dword *) 0xE000E290)) // NVIC_ICPR RW 0x00000000 Interrupt Clear-Pending Register on page 4-4.
#define NVIC_ICPR5  (*((volatile dword *) 0xE000E294)) // NVIC_ICPR RW 0x00000000 Interrupt Clear-Pending Register on page 4-4.
#define NVIC_ICPR6  (*((volatile dword *) 0xE000E298)) // NVIC_ICPR RW 0x00000000 Interrupt Clear-Pending Register on page 4-4.
#define NVIC_ICPR7  (*((volatile dword *) 0xE000E29C)) // NVIC_ICPR RW 0x00000000 Interrupt Clear-Pending Register on page 4-4.

#define NVIC_IPR0   (*((volatile dword *) 0xE000E400)) // 0..3 Interrupt Priority Registers on page 4-5.
#define NVIC_IPR1   (*((volatile dword *) 0xE000E404)) // 4..7
#define NVIC_IPR2   (*((volatile dword *) 0xE000E408)) // 8..11
#define NVIC_IPR3   (*((volatile dword *) 0xE000E40C)) // 12..15
#define NVIC_IPR4   (*((volatile dword *) 0xE000E410)) // 16..19
#define NVIC_IPR5   (*((volatile dword *) 0xE000E414)) // 20..23
#define NVIC_IPR6   (*((volatile dword *) 0xE000E418)) // 24..27
#define NVIC_IPR7   (*((volatile dword *) 0xE000E41C)) // 28..31

#define NVIC_IPR8   (*((volatile dword *) 0xE000E420)) // 32..35
#define NVIC_IPR9   (*((volatile dword *) 0xE000E424)) // 36..39
#define NVIC_IPR10  (*((volatile dword *) 0xE000E428)) // 40..43
#define NVIC_IPR11  (*((volatile dword *) 0xE000E42C)) // 44..47
#define NVIC_IPR12  (*((volatile dword *) 0xE000E430)) // 48..51
#define NVIC_IPR13  (*((volatile dword *) 0xE000E434)) // 52..55
#define NVIC_IPR14  (*((volatile dword *) 0xE000E438)) // 56..59
#define NVIC_IPR15  (*((volatile dword *) 0xE000E43C)) // 60..63
#define NVIC_IPR16  (*((volatile dword *) 0xE000E440)) // 64..67
#define NVIC_IPR17  (*((volatile dword *) 0xE000E444)) // 68..71
#define NVIC_IPR18  (*((volatile dword *) 0xE000E448)) // 72..75
// und weiter bis IPR59



/* Systick  */
/* Max Count = 1E24 -1,
   in SYSTICK_CTRL:
   Bit  0 = Enable,
   Bit  1 = Enable Interrupt,
   Bit  3 = Clocksource: Hi --> HCLK, Lo --> HCLK/8
   Bit 16 = wir dbei jedem Reload gesetzt, per Soft zurckzustellen
 */
#define SYSTICK_CTRL_ENABLE     1           /* Counter enable */
#define SYSTICK_CTRL_TICKINT    2           /* Counting down to 0 pends the SysTick handler */
#define SYSTICK_CTRL_CLKSOURCE  4           /* Clock source */
#define SYSTICK_CTRL_COUNTFLAG  0x00010000  /* Count Flag */

#define SYSTICK_CTRL     (*(volatile unsigned long*) 0xE000E010)   /* SysTick Control and Status Register */
#define SYSTICK_LOAD     (*(volatile unsigned long*) 0xE000E014)   /* SysTick Reload Value Register       */
#define SYSTICK_VAL      (*(volatile unsigned long*) 0xE000E018)   /* SysTick Current Value Register      */
#define SYSTICK_CALIB    (*(volatile unsigned long*) 0xE000E01C)   /* SysTick Calibration Register        */




/* Interrupt-Nummern */

/******  Cortex-M3 Processor Exceptions Numbers *****/
#define  NonMaskableInt_IRQn         -14     /*  2 Non Maskable Interrupt */
#define  MemoryManagement_IRQn       -12     /*  4 Cortex-M3 Memory Management Interrupt  */
#define  BusFault_IRQn               -11     /*  5 Cortex-M3 Bus Fault Interrupt     */
#define  UsageFault_IRQn             -10     /*  6 Cortex-M3 Usage Fault Interrupt   */
#define  SVCall_IRQn                  -5     /* 11 Cortex-M3 SV Call Interrupt       */
#define  DebugMonitor_IRQn            -4     /* 12 Cortex-M3 Debug Monitor Interrupt */
#define  PendSV_IRQn                  -2     /* 14 Cortex-M3 Pend SV Interrupt       */
#define  SysTick_IRQn                 -1     /* 15 Cortex-M3 System Tick Interrupt   */

/******  STM32 specific Interrupt Numbers *********************************************************/
#define  WWDG_IRQn                    0      /* Window WatchDog Interrupt                            */
#define  PVD_IRQn                     1      /* PVD through EXTI Line detection Interrupt            */
#define  TAMPER_IRQn                  2      /* Tamper Interrupt                                     */
#define  RTC_IRQn                     3      /* RTC global Interrupt                                 */
#define  FLASH_IRQn                   4      /* FLASH global Interrupt                               */
#define  RCC_IRQn                     5      /* RCC global Interrupt                                 */
#define  EXTI0_IRQn                   6      /* EXTI Line0 Interrupt                                 */
#define  EXTI1_IRQn                   7      /* EXTI Line1 Interrupt                                 */
#define  EXTI2_IRQn                   8      /* EXTI Line2 Interrupt                                 */
#define  EXTI3_IRQn                   9      /* EXTI Line3 Interrupt                                 */
#define  EXTI4_IRQn                   10     /* EXTI Line4 Interrupt                                 */
#define  DMA1_Channel1_IRQn           11     /* DMA1 Channel 1 global Interrupt                      */
#define  DMA1_Channel2_IRQn           12     /* DMA1 Channel 2 global Interrupt                      */
#define  DMA1_Channel3_IRQn           13     /* DMA1 Channel 3 global Interrupt                      */
#define  DMA1_Channel4_IRQn           14     /* DMA1 Channel 4 global Interrupt                      */
#define  DMA1_Channel5_IRQn           15     /* DMA1 Channel 5 global Interrupt                      */
#define  DMA1_Channel6_IRQn           16     /* DMA1 Channel 6 global Interrupt                      */
#define  DMA1_Channel7_IRQn           17     /* DMA1 Channel 7 global Interrupt                      */
#define  ADC1_2_IRQn                  18     /* ADC1 et ADC2 global Interrupt                        */
#define  USB_HP_CAN1_TX_IRQn          19     /* USB High Priority or CAN1 TX Interrupts              */
#define  USB_LP_CAN1_RX0_IRQn         20     /* USB Low Priority or CAN1 RX0 Interrupts              */
#define  CAN1_RX1_IRQn                21     /* CAN1 RX1 Interrupt                                   */
#define  CAN1_SCE_IRQn                22     /* CAN1 SCE Interrupt                                   */
#define  EXTI9_5_IRQn                 23     /* External Line[9:5] Interrupts                        */
#define  TIM1_BRK_IRQn                24     /* TIM1 Break Interrupt                                 */
#define  TIM1_UP_IRQn                 25     /* TIM1 Update Interrupt                                */
#define  TIM1_TRG_COM_IRQn            26     /* TIM1 Trigger and Commutation Interrupt               */
#define  TIM1_CC_IRQn                 27     /* TIM1 Capture Compare Interrupt                       */
#define  TIM2_IRQn                    28     /* TIM2 global Interrupt                                */
#define  TIM3_IRQn                    29     /* TIM3 global Interrupt                                */
#define  TIM4_IRQn                    30     /* TIM4 global Interrupt                                */
#define  I2C1_EV_IRQn                 31     /* I2C1 Event Interrupt                                 */
#define  I2C1_ER_IRQn                 32     /* I2C1 Error Interrupt                                 */
#define  I2C2_EV_IRQn                 33     /* I2C2 Event Interrupt                                 */
#define  I2C2_ER_IRQn                 34     /* I2C2 Error Interrupt                                 */
#define  SPI1_IRQn                    35     /* SPI1 global Interrupt                                */
#define  SPI2_IRQn                    36     /* SPI2 global Interrupt                                */
#define  USART1_IRQn                  37     /* USART1 global Interrupt                              */
#define  USART2_IRQn                  38     /* USART2 global Interrupt                              */
#define  USART3_IRQn                  39     /* USART3 global Interrupt                              */
#define  EXTI15_10_IRQn               40     /* External Line[15:10] Interrupts                      */
#define  RTCAlarm_IRQn                41     /* RTC Alarm through EXTI Line Interrupt                */
#define  USBWakeUp_IRQn               42     /* USB WakeUp from suspend through EXTI Line Interrupt  */
#define  TIM8_BRK_IRQn                43     /* TIM8 Break Interrupt                                 */
#define  TIM8_UP_IRQn                 44     /* TIM8 Update Interrupt                                */
#define  TIM8_TRG_COM_IRQn            45     /* TIM8 Trigger and Commutation Interrupt               */
#define  TIM8_CC_IRQn                 46     /* TIM8 Capture Compare Interrupt                       */
#define  ADC3_IRQn                    47     /* ADC3 global Interrupt                                */
#define  FSMC_IRQn                    48     /* FSMC global Interrupt                                */
#define  SDIO_IRQn                    49     /* SDIO global Interrupt                                */
#define  TIM5_IRQn                    50     /* TIM5 global Interrupt                                */
#define  SPI3_IRQn                    51     /* SPI3 global Interrupt                                */
#define  UART4_IRQn                   52     /* UART4 global Interrupt                               */
#define  UART5_IRQn                   53     /* UART5 global Interrupt                               */
#define  TIM6_IRQn                    54     /* TIM6 global Interrupt                                */
#define  TIM7_IRQn                    55     /* TIM7 global Interrupt                                */
#define  DMA2_Channel1_IRQn           56     /* DMA2 Channel 1 global Interrupt                      */
#define  DMA2_Channel2_IRQn           57     /* DMA2 Channel 2 global Interrupt                      */
#define  DMA2_Channel3_IRQn           58     /* DMA2 Channel 3 global Interrupt                      */
#define  DMA2_Channel4_5_IRQn         59     /* DMA2 Channel 4 and Channel 5 global Interrupt        */



#endif

