/*******************************************************
  Events.h   Unit fr die Ereignisverwaltung
********************************************************/

#ifndef EVENTSINCLUDED
#define EVENTSINCLUDED

/*
Ein Event wird durch ein dword (32bit) dargestellt.
Davon
die untersten 4 Bit     (0..3)   sind die Event-Gruppe,
die nchsthheren 8 Bit (4..11)  sind Event-ID's innerhalb der Gruppe
die brigen 20 Bits     (12.. 31) sind Zusatz-Informationen
*/

#define evGruppenMaske  0x00F
#define evEventMaske    0xFFF

/*
Zur Klarstellung:
Alle diese Events sind mehr oder weniger Beispiele, um zu zeigen wie man es macht.
Fr ein konkretes Projekt definiert man sich seine eigenen Events, die zur Aufgabe passen.


Hier, zwecks Systematik:
  0 = nix, Event gelscht (das sollte IMMER so gelten)
  1 = Hardware-Events  (von Treibern gesendet)
  2 = Tastatur-Events  (Tasten gedrckt oder losgelassen)
  3 = Maus-Events      (Position und Tasten)
  4 = Men-Events      (meninterner Broadcast, fr men-interne Organisation)
  5 = Timer-Events     (Zeitsignale, Botschaften, etc.)
  6 = Org-Events       (Organisation und Broadcast aus dem Betriebssystem)
  7 = Sensor-Events    (Positionssensoren usw.)
  8 = Error-Events     (alles, was mit Fehlern zu tun hat)
  9..14 noch undefiniert
 15 = App-Events       (anwendungsspezifisches Zeugs)
*/

/* die Gruppen-Id's */
#define idHard    1
#define idTast    2
#define idMaus    3
#define idMenu    4
#define idTimer   5
#define idOrg     6
#define idSensor  7
#define idError   8
#define idApp    15


/* Gruppe 1 = Hardware-ID's */
#define idDrehL       ((1<<4)  | idHard)
#define idDrehR       ((2<<4)  | idHard)
#define idDrehT       ((3<<4)  | idHard)
#define idLTast       ((4<<4)  | idHard)
#define idRTast       ((5<<4)  | idHard)
#define idIrAvail     ((6<<4)  | idHard)


/* Gruppe 2 = Tasten-ID's   */
#define idKbMake      ((0<<4) | idTast)
#define idKbBreak     ((1<<4) | idTast)
/* hierbei: Bit 16..31 ist der WideChar dazu */
#define idEnterTaste  ((2<<4) | idTast)
#define idLinksTaste  ((3<<4) | idTast)
#define idRechtsTaste ((4<<4) | idTast)
#define idRaufTaste   ((5<<4) | idTast)
#define idRunterTaste ((6<<4) | idTast)
#define idEscapeTaste ((7<<4) | idTast)

#define idTastenFeld  ((8<<4)  | idTast)
#define idTaste0      ((8<<4)  | idTast)
#define idTaste1      ((9<<4)  | idTast)
#define idTaste2      ((10<<4) | idTast)
#define idTaste3      ((11<<4) | idTast)
#define idTaste4      ((12<<4) | idTast)
#define idTaste5      ((13<<4) | idTast)
#define idTaste6      ((14<<4) | idTast)
#define idTaste7      ((15<<4) | idTast)
#define idTaste8      ((16<<4) | idTast)
#define idTaste9      ((17<<4) | idTast)  /* usw. bis 4095 je nach Bedarf */


/* Gruppe 3 = Maus-ID's     */
/* Bits (4..7 )  = Tasten
   Bits (8..19)  = X Koordinate (12 Bit)
   Bits (20..31) = Y Koordinate (12 Bit)
*/


/* Gruppe 4 = Menu-ID's     */
#define idMenuInit      ((0<<4) | idMenu)
#define idMenuClear     ((1<<4) | idMenu)
#define idMenuRedraw    ((2<<4) | idMenu)

#define idMenuSelect1   ((201<<4) | idMenu)
#define idMenuSelect2   ((202<<4) | idMenu)
#define idMenuSelect3   ((203<<4) | idMenu)   /* usw. je nach Bedarf */


/* Gruppe 5 = Timer-ID's   */
#define idSekundeUm     ((0<<4)  | idTimer)


/* Gruppe 6 = Org-ID's      */
#define idBinFertig    ((1<<4)  | idOrg)      // Fertig-Meldung
#define idNullpunkt    ((2<<4)  | idOrg)      // Fertig-Meldung
#define idMessbereit   ((3<<4)  | idOrg)      // Fertig-Meldung


/* Gruppe 7 = Sensor-ID's      */
#define idSens1        ((1<<4) | idSensor)   // Dreh-Null-Sensor durchfahren
#define idSens2        ((2<<4) | idSensor)   // Dreh-Ende-Sensor durchfahren
#define idSens3        ((3<<4) | idSensor)
#define idSens4        ((4<<4) | idSensor)
#define idSens5        ((5<<4) | idSensor)
#define idSens6        ((6<<4) | idSensor)
#define idSens7        ((7<<4) | idSensor)


/* Gruppe 8 = Error-ID's      */
#define idNoInit        ((1<<4) | idError)
#define idTimeout       ((2<<4) | idError)
#define idPosErr        ((3<<4) | idError)


/* ladbare Timer, d.h. Down-Counters, die bei Erreichen von 0 einen Event
 in den Eventbuffer einspeisen  */

#define DTTE  struct TTimerEvent
struct TTimerEvent
{ long  endzeit;
  dword aEvent;
};

#define numtimers   6
extern DTTE CurrTimers[numtimers];






/******* Funktionen *****************/

extern void  Event_Init          (void);
extern bool  Event_avail         (void);
extern bool  Exists_Event        (dword aEvent);
extern void  Kill_Event          (dword aEvent);
extern dword Get_Event           (void);
extern void  Add_Event           (dword aEvent);
extern bool  Add_Delayed_Event   (dword millisec, dword aEvent);
extern void  Kill_Delayed_Event  (dword aEvent);
extern bool  Exists_Delayed_Event(dword aEvent);

#endif
