/*******************************************************
  SysTick: Unit fr die Zeitverwaltung
*******************************************************/

#include "StdTypes.h"
#include "STM32F103all.h"
#include "startup.h"
#include "config.h"
#include "events.h"
#include "systick.h"

static   volatile byte  iTicks;
static   volatile byte  SecTicks;

/*
Als innere Uhr wird der Cortex-Systick benutzt, der alle Millisekunde 1x berluft.
Die Variable Ticks wird dabei um 1 erhht, ihr Stand ist also gleichbedeutend mit der Uhrzeit in Millisekunden.
*/

void  __irq  SysTick_Handler (void)
{ int i;

  ++Ticks;
  ++iTicks;

  if (iTicks<10) return;      /* der Rest nur alle 10 ms */

  if (Ticks >= 86400000)      /* wenn 1 Tag um ist */
  { Ticks = 0;
    i = 0;
    while (i<numtimers)
    { if (CurrTimers[i].aEvent)
          CurrTimers[i].endzeit = CurrTimers[i].endzeit - 86400000;
      ++i;
    }
  }
  iTicks = 0;
  ++SecTicks;

  if (SecTicks >= 100)
     { SecTicks = 0; Add_Event(idSekundeUm); }

  /* ladbare Timer bearbeiten */
   i = 0;
   while (i<numtimers)
   { if (CurrTimers[i].aEvent)
     { if (CurrTimers[i].endzeit < Ticks)    /* ob abgelaufen */
       {                                    /* hier ggf. Flags auswerten */
         Add_Event(CurrTimers[i].aEvent);
         CurrTimers[i].aEvent = 0;
       }
     }
     ++i;
   }
}



/* Cortex M4 Systick als System-Uhr einrichten,
   der Systemtakt ist 100 MHz,
   und alle 1 ms sollte ein Interrupt erfolgen.
 */
void Systemtick_Init (void)
{  Ticks = iTicks = SecTicks = 0;
   SYST_RVR = (F_Platform/1000)-1;      // 1 ms
   SYST_CSR = 7;                        // enabled, cclk als Takt
}


void Warte_ms (long Millisekunden)
{ dword D;
  long  L;

  D = Ticks;
  L = Millisekunden;
  while (L>0)
  { if (D!=Ticks)
    { --L;
      D = Ticks;
    }
  }
}


   /* ende */
